import itertools
from collections import deque

import matplotlib.animation as animation
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.widgets import Button, Slider
from scipy.integrate import solve_ivp

from logslider import LogSlider0

# d²x/dt² + ω₀ x + 2β dx/dt + 2γ² x³ = A cos(ωt + φ)
ω0 = 1.0
β = 0.05
γ = 2.0
A = 6.0
ω = 1.6
φ = 0.0

x0 = 0.
v0 = 0.

t_stop = 120  # how many seconds to simulate


def derivs(t, x):
    dx = np.array([x[1], A * np.cos(ω * t + φ) - 2. * β * x[1] - ω0**2 * x[0] - 2. * γ**2 * x[0]**3])
    return dx


# create a time array from 0..t_stop sampled at 0.02 second steps
dt = 0.01
t = np.arange(0, t_stop, dt)


def solve():
    xx = solve_ivp(derivs, t[[0, -1]], np.array([x0, v0], dtype=float), t_eval=t, method='DOP853', rtol=1e-6).y.T
    xmin, xmax = np.min(xx[:, 0]), np.max(xx[:, 0])
    dx = (xmax - xmin) * 0.05
    xmin -= dx
    xmax += dx
    ymin, ymax = np.min(xx[:, 1]), np.max(xx[:, 1])
    dy = (ymax - ymin) * 0.05
    ymin -= dy
    ymax += dy
    return xx, (xmin, xmax), (ymin, ymax)


xx, xlim, ylim = solve()

fig = plt.figure(figsize=(5, 4))
ax = fig.add_subplot(autoscale_on=False, xlim=xlim, ylim=ylim)

manager = fig.canvas.manager
manager.window.findChild(QtWidgets.QToolBar).setVisible(False)
manager.set_window_title("Dudnienia")
manager.full_screen_toggle()

ax.axhline(0, color='k', lw=0.7)
ax.axvline(0, color='k', lw=0.7)
point, = ax.plot([], [], 'o', lw=3)
trace, = ax.plot([], [], '-', lw=1, ms=2)
ax.set_xlabel("$x$")
ax.set_ylabel("$dx/dt$")
history_x, history_y = deque(), deque()

fig.subplots_adjust(bottom=0.18, left=0.07)
βax = fig.add_axes([0.15, 0.07, 0.25, 0.03])
γax = fig.add_axes([0.15, 0.04, 0.25, 0.03])
Aax = fig.add_axes([0.15, 0.01, 0.25, 0.03])
ωax = fig.add_axes([0.55, 0.07, 0.25, 0.03])
xax = fig.add_axes([0.55, 0.04, 0.25, 0.03])
vax = fig.add_axes([0.55, 0.01, 0.25, 0.03])

Cax = fig.add_axes([0.85, 0.01, 0.1, 0.1])

sst = {
    'track_color': '0.7',
    'color': '0.7',
    'initcolor': 'black',
}

βslider = LogSlider0(ax=βax, label='$\\beta$', valmin=-2, valmax=1., valstep=0.01, valinit=np.log10(ω0), **sst)
γslider = Slider(ax=γax, label='$\\gamma$', valmin=0.0, valmax=10.0, valinit=γ, valstep=0.1, **sst)
Aslider = Slider(ax=Aax, label='$A$', valmin=0.0, valmax=10.0, valinit=A, valstep=0.5, **sst)
ωslider = Slider(ax=ωax, label='$\\omega$', valmin=0.0, valmax=2.0, valinit=ω0, valstep=0.01, **sst)
xslider = Slider(ax=xax, label='$x_0$', valmin=0., valmax=10.0, valstep=0.1, valinit=x0, **sst)
vslider = Slider(ax=vax, label='$v_0$', valmin=-1.0, valmax=1.0, valstep=0.25, valinit=0., **sst)

Cbutton = Button(ax=Cax, label='Commit', color='0.7', hovercolor='0.9')

βslider.set_val(np.log10(β))


def animate(i):
    x = xx[i]

    if i == 0:
        history_x.clear()
        history_y.clear()

    history_x.appendleft(x[0])
    history_y.appendleft(x[1])

    point.set_data([x[0]], [x[1]])
    trace.set_data(history_x, history_y)
    return trace, point


ani = animation.FuncAnimation(fig, animate, len(xx), interval=dt * 10, blit=True)


def commit(_):
    global β, γ, A, ω, x0, v0, xx
    β = βslider.val
    γ = γslider.val
    A = Aslider.val
    ω = ωslider.val
    x0 = xslider.val
    v0 = vslider.val
    xx, xlim, ylim = solve()
    ani.frame_seq = ani.new_frame_seq()
    ax.set_xlim(*xlim)
    ax.set_ylim(*ylim)
    fig.canvas.draw_idle()


Cbutton.on_clicked(commit)

plt.show()
